/*
 *    functions for in and output
 */


#include "ct1.h"
#include "inout.h"
#include <string.h>
#include <unistd.h>
#include "gdk/gdkprivate.h"

int
save_mol (char *filename, int partial)
/* saves the current structure to a file in native chemtool format */
{
  int d;
  int w, h;
  struct data *hp_b;
  struct dc *hp_a;
  struct spline *hp_sp;
  FILE *fp;
  int mbonds = 0, mlabels = 0, msplines = 0;

  if ((fp = fopen (filename, "w")) == NULL)
    return (1);
/* get true extents of drawing*/
  w = 0;
  h = 0;
  hp_b = da_root.next;
  for (d = 0; d < hp.n; d++)
    {
      if (partial == 1 && hp_b->smarked == 0 && hp_b->tmarked == 0)
	{
	  hp_b = hp_b->next;
	}
      else
	{
	  if (partial)
	    mbonds++;
	  w = MAX (w, hp_b->x);
	  w = MAX (w, hp_b->tx);
	  h = MAX (h, hp_b->y);
	  h = MAX (h, hp_b->ty);
	  hp_b = hp_b->next;
	}
    }
  hp_a = dac_root.next;
  for (d = 0; d < hp.nc; d++)
    {
      if (partial && !hp_a->marked)
	{
	  hp_a = hp_a->next;
	}
      else
	{
	  if (partial)
	    mlabels++;
	  w = MAX (w, hp_a->x);
	  h = MAX (h, hp_a->y);
	  hp_a = hp_a->next;
	}
    }

  hp_sp = sp_root.next;
  for (d = 0; d < hp.nsp; d++)
    {
      if (partial && !hp_sp->marked)
	{
	  hp_sp = hp_sp->next;
	}
      else
	{
	  if (partial)
	    msplines++;
	  w = MAX (w, hp_sp->x0);
	  h = MAX (h, hp_sp->y0);
	  w = MAX (w, hp_sp->x1);
	  h = MAX (h, hp_sp->y1);
	  w = MAX (w, hp_sp->x2);
	  h = MAX (h, hp_sp->y2);
	  w = MAX (w, hp_sp->x3);
	  h = MAX (h, hp_sp->y3);
	  hp_sp = hp_sp->next;
	}
    }
    
  w = w * 1.1;
  h = h * 1.1;
  fprintf (fp, "Chemtool Version " VERSION "\n");

  fprintf (fp, "geometry %i %i\n", w, h);

  hp_b = da_root.next;
  if (partial)
    fprintf (fp, "bonds %i\n", mbonds);
  else
    fprintf (fp, "bonds %i\n", hp.n);
  for (d = 0; d < hp.n; d++)
    {
      if (partial && !hp_b->smarked && !hp_b->tmarked)
	{
	  hp_b = hp_b->next;
	}
      else
	{
	  fprintf (fp, "%i	%i	%i	%i	%i\n",
		   hp_b->x, hp_b->y, hp_b->tx, hp_b->ty, hp_b->bond);
	  hp_b = hp_b->next;
	}
    }

  hp_a = dac_root.next;
  if (partial)
    fprintf (fp, "atoms %i\n", mlabels);
  else
    fprintf (fp, "atoms %i\n", hp.nc);
  for (d = 0; d < hp.nc; d++)
    {
      if (partial && !hp_a->marked)
	{
	  hp_a = hp_a->next;
	}
      else
	{
	  fprintf (fp, "%i	%i	%s	%i\n",
		   hp_a->x, hp_a->y, hp_a->c, hp_a->direct);
	  hp_a = hp_a->next;
	}
    }

  hp_sp = sp_root.next;
  if (partial)
    fprintf (fp, "splines %i\n", msplines);
  else
    fprintf (fp, "splines %i\n", hp.nsp);
  for (d = 0; d < hp.nsp; d++)
    {
      if (partial && !hp_sp->marked)
	{
	  hp_sp = hp_sp->next;
	}
      else
	{
	  fprintf (fp, "%i	%i	%i	%i	%i	%i	%i	%i	%i\n",
		   hp_sp->x0, hp_sp->y0, hp_sp->x1, hp_sp->y1, 
		   hp_sp->x2, hp_sp->y2, hp_sp->x3, hp_sp->y3,
		   hp_sp->type); 
	  hp_sp = hp_sp->next;
	}
    }

  fclose (fp);
  return (0);
}

int
load_mol (char *filename)
/* loads a chemtool file, checking for correct header */
{
  int d, n;
  int x, y, tx, ty, x2,y2, x3, y3, b;
  char str[255], str1[255];
  FILE *fp;
  char version[10];

  if ((fp = fopen (filename, "r")) == NULL)
    return (1);

  fscanf (fp, "%s %s %s", str, str1, version);
  if (strcmp (str, "Chemtool") || strcmp (str1, "Version"))
    return (2);

  clear_data ();

  fscanf (fp, "%s %i %i", str, &x, &y);
  if (!strcmp (str, "geometry"))
    {
      head.width = MAX (head.width, x);
      head.height = MAX (head.height, y);
    }

  fscanf (fp, "%s %i", str, &n);
  if (!strcmp (str, "bonds") || !strcmp (str, "bounds"))
    {				/* typo in versions < 1.1.2 */
      for (d = 0; d < n; d++)
	{
	  fscanf (fp, "%i %i %i %i %i", &x, &y, &tx, &ty, &b);
	  add_struct (x, y, tx, ty, b, 0, 0);
	}
    }

  fscanf (fp, "%s %i", str, &n);
  if (!strcmp (str, "atoms"))
    {
      for (d = 0; d < n; d++)
	{
	  fscanf (fp, "%i %i %s %i", &x, &y, str, &b);
	  add_char (x, y, str, b, 0);
	}
    }

  fscanf (fp, "%s %i", str, &n);
  if (!strcmp (str, "splines"))
    {
      for (d = 0; d < n; d++)
	{
	  fscanf (fp, "%i %i %i %i %i %i %i %i %i", &x, &y, &tx, &ty,
	  &x2, &y2, &x3, &y3, &b);
	  add_spline (x, y, tx, ty, x2, y2, x3, y3, b, 0);
	}
    }

  fclose (fp);
  if (atof (version) > atof (VERSION))
    return (3);
  return (0);
}

int
add_mol (char *filename)
/* loads a chemtool file and adds its contents at a given position */
{
  int d, n;
  int x, y, tx, ty, x2, y2, x3, y3, b;
  int xdiff = 0, ydiff = 0;
  char str[255];
  FILE *fp;
  char version[10];

  if ((fp = fopen (filename, "r")) == NULL)
    return (1);

  fscanf (fp, "%s %s %s", str, str, version);
  if (atof (version) > atof (VERSION))
    return (2);


  fscanf (fp, "%s %i %i", str, &x, &y);
  if (!strcmp (str, "geometry"))
    {
      head.width += x;
      mark.w = x;
      head.height += y;
      mark.h = y;
    }

  fscanf (fp, "%s %i", str, &n);
  if (!strcmp (str, "bonds") || !strcmp (str, "bounds"))
    {
      for (d = 0; d < n; d++)
	{
	  fscanf (fp, "%i %i %i %i %i", &x, &y, &tx, &ty, &b);
	  if (d == 0)
	    {
	      xdiff = mark.x - x;
	      ydiff = mark.y - y;
	      xref = x + xdiff;
	      yref = y + ydiff;
	    }
	  x = x + xdiff;
	  y = y + ydiff;
	  tx = tx + xdiff;
	  ty = ty + ydiff;
	  add_struct (x, y, tx, ty, b, 1, 1);
	}
    }

  fscanf (fp, "%s %i", str, &n);
  if (!strcmp (str, "atoms"))
    {
      for (d = 0; d < n; d++)
	{
	  fscanf (fp, "%i %i %s %i", &x, &y, str, &b);
	  x = x + xdiff;
	  y = y + ydiff;
	  add_char (x, y, str, b, 1);
	}
    }

  fscanf (fp, "%s %i", str, &n);
  if (!strcmp (str, "splines"))
    {
      for (d = 0; d < n; d++)
	{
	  fscanf (fp, "%i %i %i %i %i %i %i %i %i", &x, &y, &tx, &ty,
	  &x2, &y2, &x3, &y3, &b);
	  add_spline (x, y, tx, ty, x2, y2, x3, y3, b, 0);
	}
    }

  fclose (fp);
  return (0);
}


int
export_bitmap (char *filename)
/* exports the current drawing to an X11 bitmap file */
{
  int x, y, w, h;
  struct data *hp_b;
  struct dc *hp_a;
  struct spline *hp_sp;
  int d;
  int retval;
  char xfile[512];
  Pixmap buffer;
  GdkWindowPrivate *drawable_private;
  GdkWindowPrivate *src_private;
  GdkPixmap *bitmap;
  GdkGC *bitmapgc;
  GdkGCPrivate *gc_private;

  if (mark.flag)
    {
      x = mark.x * size_factor + 3;
      y = mark.y * size_factor + 3;
      w = mark.w * size_factor - 6;
      h = mark.h * size_factor - 6;
    }
  else
    {
/* get true extents of drawing*/
  x = 10000;
  y = 10000;
  w = 0;
  h = 0;
  hp_b = da_root.next;
  for (d = 0; d < hp.n; d++)
    {
	  w = MAX (w, hp_b->x);
	  w = MAX (w, hp_b->tx);
	  h = MAX (h, hp_b->y);
	  h = MAX (h, hp_b->ty);
	  x = MIN (x, hp_b->x);
	  x = MIN (x, hp_b->tx);
	  y = MIN (y, hp_b->y);
	  y = MIN (y, hp_b->ty);
	  hp_b = hp_b->next;
	}
  hp_a = dac_root.next;
  for (d = 0; d < hp.nc; d++)
    {
	  w = MAX (w, hp_a->x);
	  h = MAX (h, hp_a->y);
	  x = MIN (x, hp_a->x);
	  y = MIN (y, hp_a->y);
	  hp_a = hp_a->next;
    }

  hp_sp = sp_root.next;
  for (d = 0; d < hp.nsp; d++)
    {
	  w = MAX (w, hp_sp->x0);
	  h = MAX (h, hp_sp->y0);
	  w = MAX (w, hp_sp->x1);
	  h = MAX (h, hp_sp->y1);
	  w = MAX (w, hp_sp->x2);
	  h = MAX (h, hp_sp->y2);
	  w = MAX (w, hp_sp->x3);
	  h = MAX (h, hp_sp->y3);
	  x = MIN (x, hp_sp->x0);
	  y = MIN (y, hp_sp->y0);
	  x = MIN (x, hp_sp->x1);
	  y = MIN (y, hp_sp->y1);
	  x = MIN (x, hp_sp->x2);
	  y = MIN (y, hp_sp->y2);
	  x = MIN (x, hp_sp->x3);
	  y = MIN (y, hp_sp->y3);
	  hp_sp = hp_sp->next;
	}
  x = x*size_factor*0.9;
  y = y*size_factor*0.9;
  w = w*size_factor*1.1-x;
  h = h*size_factor*1.1-y;
   }
  drawable_private = (GdkWindowPrivate *) drawing_area;
  src_private = (GdkWindowPrivate *) picture;
  bitmap = gdk_pixmap_new (drawing_area->window, 1, 1, 1);
  bitmapgc = gdk_gc_new (bitmap);
  gc_private = (GdkGCPrivate *) bitmapgc;
  buffer = XCreatePixmap (src_private->xdisplay, src_private->xwindow,
			  w, h, 1);
  XCopyPlane (src_private->xdisplay, src_private->xwindow, buffer,
	      gc_private->xgc, x, y, w, h, 0, 0, 1);

  if (strlen (filename))
    {
      sprintf (xfile, "%s", filename);
      retval = XWriteBitmapFile (GDK_DISPLAY (), xfile, buffer, w, h, -1, -1);
      gdk_pixmap_unref (bitmap);
      gdk_gc_unref (bitmapgc);
      XFreePixmap (GDK_DISPLAY (), buffer);
      return (retval);
    }
  return (0);
}


void
xfig_line (FILE * fp, int x, int y, int tx, int ty)
/* writes a single line bond in XFig notation */
{
  fprintf (fp, "%i %i %i %i %i %i %i %i %i %f %i %i %i %i %i %i\n",
	   figline.object, figline.sub_type, figline.line_style,
	   figline.thickness, figline.pen_color, figline.fill_color,
	   figline.depth, figline.pen_style, figline.area_fill,
	   figline.style_val, figline.join_style, figline.cap_style,
	   figline.radius, figline.forward_arrow, figline.backward_arrow,
	   figline.npoints);
  fprintf (fp, "%i %i %i %i\n", x, y, tx, ty);
}

void
xfig_wedge (FILE * fp, int x, int y, int tx, int ty, int factor, int latex)
/* writes a wedge definition in XFig notation */
{
struct data *hpc;
struct xy_co *coord;
int d,x1,y1,x2,y2;
int area;

	   x1 = tx - (int) (0.1 * (float) (ty - y));
	   y1 = ty + (int) (0.1 * (float) (tx - x));
	   x2 = tx + (int) (0.1 * (float) (ty - y));
	   y2 = ty - (int) (0.1 * (float) (tx - x));
  hpc = da_root.next;
  for (d = 0; d < hp.n; d++)
    {
      if (hpc->bond == 10){
      if ( (fabs(hpc->x*factor - tx)<3 && fabs(hpc->y*factor-ty) <3) 
	|| (fabs(hpc->tx*factor-tx)<3 && fabs(hpc->ty*factor-ty) <3) ){
  coord = center_double_bond (hpc->x, hpc->y, hpc->tx, hpc->ty, 4);
      if (fabs(hpc->x*factor - tx)<3 && fabs(hpc->y*factor-ty) <3)  {
  x1 = coord->x *factor;
  y1 = coord->y *factor;
  coord++;
  x2 = coord->x *factor;
  y2 = coord->y *factor;
  }else{
  x1 = coord->tx *factor;
  y1 = coord->ty *factor;
  coord++;
  x2 = coord->tx *factor;
  y2 = coord->ty *factor;
      }
            area = 0.5*fabs(x*(y1-y2)
                      +x1*(y2-y)
                      +x2*(y-y1));
          
          if (fabs(area) < 76*factor) {
	   x1 = tx - (int) (0.05* (float)(ty-y));
	   y1 = ty + (int) (0.05 * (float) (tx - x));
	   x2 = tx + (int) (0.05 * (float) (ty - y));
	   y2 = ty - (int) (0.05 * (float) (tx - x));
      }

      }
  }
      hpc = hpc->next;
  }

if (!latex){
  fprintf (fp, "%i %i %i %i %i %i %i %i %i %f %i %i %i %i %i %i\n",
	   figline.object, figline.sub_type, figline.line_style,
	   figline.thickness, figline.pen_color, figline.fill_color,
	   figline.depth, figline.pen_style, figline.area_fill + 1,
	   figline.style_val, figline.join_style, figline.cap_style,
	   figline.radius, figline.forward_arrow, figline.backward_arrow,
	   figline.npoints + 1);
  fprintf (fp, "%i %i %i %i %i %i\n", x, y, x1, y1, x2, y2);
 }     else  {
       for (d=0;d<10;d++)
           xfig_line(fp,x,y,x1 + d*(x2-x1)/10, y1 + d*  (y2-y1)/10);
       }

}

void
xfig_wiggly (FILE * fp, int x, int y, int tx, int ty)
/* writes a wavy bond in XFig notation */
{
  int arccode = 5;
  int arctype = 1;
  int direction = 1 ;
  float xlen, ylen, veclen, boxlen;
  int boxcorx, boxcory;
  int i;
  float center_x, center_y;
  int xstart, ystart, xmid, ymid, xend, yend;
  xlen = tx - x;
  ylen = ty - y;

  veclen = calc_vector (abs (tx - x), abs (ty - y));
  boxlen = 0.2 * veclen;
  boxcorx = (1. - (float) xlen / (float) veclen) * boxlen / 2;
  boxcory = (1. - (float) ylen / (float) veclen) * boxlen / 2;

  for (i = 0; i < 5; i++)
    {
      direction = abs (direction - 1);
      center_x = x + (0.2 * i + .1) * xlen;
      center_y = y + (0.2 * i + .1) * ylen;
      xstart = x + 0.2 * i * xlen;
      ystart = y + 0.2 * i * ylen;
      xmid = center_x - boxcorx * pow (-1, i);
      ymid = center_y - boxcory * pow (-1, i);
	if (x>tx){
      xmid = center_x + boxcorx * pow (-1, i);
      ymid = center_y + boxcory * pow (-1, i);
        }
      xend = x + 0.2 * (i + 1) * xlen;
      yend = y + 0.2 * (i + 1) * ylen;

      fprintf (fp,
	       "%i %i %i %i %i %i %i %i %i %f %i %i %i %i %f %f %i %i %i %i %i %i\n",
	       arccode, arctype, figline.line_style, figline.thickness,
	       figline.pen_color, figline.fill_color, figline.depth,
	       figline.pen_style, figline.area_fill, figline.style_val,
	       figline.cap_style, direction, figline.forward_arrow,
	       figline.backward_arrow, center_x, center_y, xstart, ystart,
	       xmid, ymid, xend, yend);
    }
}

void
xfig_arrow (FILE * fp, int x, int y, int tx, int ty, int type, int latex)
/* writes an arrow in XFig notation */
{
  float xlen, ylen;
  float headfact;
  int xbase, ybase, xside, yside, xend, yend;
  int i;
  
  xlen = tx - x;
  ylen = ty - y;
  headfact = 0.8;
  xbase = x + headfact * xlen;
  ybase = y + headfact * ylen;
  xside = xbase + 0.1 * ylen;
  yside = ybase - 0.1 * xlen;
  xend = xbase - type * (0.1 * ylen);	/*type=0 (half arrow) ends on baseline, */
  yend = ybase + type * (0.1 * xlen);	/*type=1 is a symmetrical arrowhead */


  if (type == 1)
    {
      /* symmetrical arrow */
      fprintf (fp, "%i %i %i %i %i %i %i %i %i %f %i %i %i %i %i %i\n",
	       figline.object, figline.sub_type, figline.line_style,
	       figline.thickness, figline.pen_color, figline.fill_color,
	       figline.depth, figline.pen_style, figline.area_fill,
	       figline.style_val, figline.join_style, figline.cap_style,
	       figline.radius, figline.forward_arrow + 1,
	       figline.backward_arrow, figline.npoints);
      fprintf (fp, "%i %i %f %f %f\n",	/* arrow attributes */
	       1,		/* arrow_type (triangle, as before) */
	       1,		/* arrow_style (1 == filled) */
	       2.00,		/* arrow_thickness (1/80inch) */
	       72.00,		/* arrow_width (fig units (?) ) */
	       114.00		/* arrow_height (fig units) */
	);
      fprintf (fp, "%i %i %i %i\n", x, y, tx, ty);
    }

  else
    {
      /* half arrow */
      /* the line */
      fprintf (fp, "%i %i %i %i %i %i %i %i %i %f %i %i %i %i %i %i\n",
	       figline.object, figline.sub_type, figline.line_style,
	       figline.thickness, figline.pen_color, figline.fill_color,
	       figline.depth, figline.pen_style, figline.area_fill,
	       figline.style_val, figline.join_style, figline.cap_style,
	       figline.radius, figline.forward_arrow, figline.backward_arrow,
	       figline.npoints);
      fprintf (fp, "%i %i %i %i\n", x, y, tx, ty);

/* and the arrowhead - a filled polyline */
if (!latex){ 
      fprintf (fp, "%i %i %i %i %i %i %i %i %i %f %i %i %i %i %i %i\n",
	       figline.object, figline.sub_type, figline.line_style,
	       figline.thickness, figline.pen_color, figline.fill_color,
	       figline.depth, figline.pen_style, figline.area_fill + 1,
	       figline.style_val, figline.join_style, figline.cap_style,
	       figline.radius, figline.forward_arrow, figline.backward_arrow,
	       figline.npoints + 1);
      fprintf (fp, "%i %i %i %i %i %i\n", tx, ty, xside, yside, xend, yend);
}else{
       for (i=0;i<10;i++)
           xfig_line(fp,tx,ty,xend + i*(xside-xend)/10, yend + i*  (yside-yend)/10);
       }

    }
}

void
xfig_circle (FILE * fp, int x, int y, int tx, int ty)
{
  int radius = calc_vector (abs (tx - x), abs (ty - y));
  float start_ang = 0.;
  int direction = 1;
  fprintf (fp, " 1 3 %i %i %i %i %i %i %i %f %i %f %i %i %i %i %i %i %i %i\n",
	   figline.line_style,
	   figline.thickness, figline.pen_color, figline.fill_color,
	   figline.depth, figline.pen_style, figline.area_fill,
	   figline.style_val, direction,
	   start_ang, x, y, radius, radius, tx, ty, tx, ty);
}

void
xfig_spline (FILE *fp, int x0, int y0, int x1, int y1, int x2, int y2, int x3, int y3,
 int type, int latex)
/* draws a spline curve, optionally with full of half arrowhead  */
{
  int xlen, ylen, xbase=0, ybase=0;
  double px0,py0,px1,py1;
  int i;
  double t;
  int xside,yside,xend,yend;

  figline.npoints=12;
	if (type==-2)
	{
	  figline.line_style = 2;
	  figline.style_val = 4;
	}
	if (type == -1 )
	{
	  figline.area_fill=0;
	  }
	if (type == 1) figline.npoints=9;

      /* the line */
      fprintf (fp, "%i %i %i %i %i %i %i %i %i %f %i %i %i %i %i %i\n",
	       figline.object, figline.sub_type, figline.line_style,
	       figline.thickness, figline.pen_color, figline.fill_color,
	       figline.depth, figline.pen_style, figline.area_fill,
	       figline.style_val, figline.join_style, figline.cap_style,
	       figline.radius, figline.forward_arrow, figline.backward_arrow,
	       figline.npoints);
	       
      fprintf (fp, "%i %i\n", x0, y0);

	  px0=x0;
	  py0=y0;
	  for (i=0;i<figline.npoints-1;i++){
	  t=(float)i/10.;
	  px1= t*t*t*(double)x3 + 3.*t*t*(1.-t)*(double)x2 + 3.*t*(1.-t)*(1.-t)*(double)x1 +(1.-t)*(1.-t)*(1.-t)*px0;
	  py1= t*t*t*(double)y3 + 3.*t*t*(1.-t)*(double)y2 + 3.*t*(1.-t)*(1.-t)*(double)y1 +(1.-t)*(1.-t)*(1.-t)*py0;
      fprintf (fp, "%i %i\n", (int)px1, (int)py1);
	if (i==7){
	xbase=px1;
	ybase=py1;
	}
	  }

	  figline.line_style = 0;
	  figline.style_val = 0;
	  figline.area_fill=-1;
	  figline.npoints = 2;
	  
    if (type <= 0) return;
    
  xlen = x3 - xbase;
  ylen = y3 - ybase;

 if (xlen != 0) xlen = (int)copysign(50.,xlen);  
 if (ylen != 0) ylen = (int)copysign(50.,ylen);                                
 
  xside = xbase + 0.5 * ylen;
  yside = ybase - 0.5 * xlen;

  if (type == 1)
    {
    
      xend = xbase - 0.5 * ylen;
      yend = ybase + 0.5 * xlen;
     px0= (xside-x0)*(xside-x0)+(yside-y0)*(yside-y0);  
     px1= (xend-x0)*(xend-x0)+(yend-y0)*(yend-y0); 
     if (px1 > px0){
        xside=xend;
        yside=yend;
        }    
      xend = xbase;	/*on baseline */
      yend = ybase;
    }
  else
    {

      xend = xbase - 0.5 * ylen;
      yend = ybase + 0.5 * xlen;
    }

/* add the arrowhead - a filled polyline */
if (!latex){
      fprintf (fp, "%i %i %i %i %i %i %i %i %i %f %i %i %i %i %i %i\n",
	       figline.object, figline.sub_type, figline.line_style,
	       figline.thickness, figline.pen_color, figline.fill_color,
	       figline.depth, figline.pen_style, figline.area_fill + 1,
	       figline.style_val, figline.join_style, figline.cap_style,
	       figline.radius, figline.forward_arrow, figline.backward_arrow,
	       figline.npoints + 1);
      fprintf (fp, "%i %i %i %i %i %i\n", (int)x3, (int)y3, xside, yside, xend, yend);
}else{
       for (i=0;i<10;i++)
           xfig_line(fp,x3,y3,xend + i*(xside-xend)/10, yend + i*  (yside-yend)/10);
       }

}



int
exfig (char *filename, int latex_flag)
/* export the current molecule in the format of Brian Smith' XFig program */
{
  struct data *hp_b;
  struct dc *hp_a;
  struct spline *hp_sp;
  struct xy_co *coord;

  FILE *fp;
  int i;
  float l;
  int d, x, y, tx, ty;
  int ulx, uly, lrx, lry;
  int ha, chl = 0;
  int fontsize = 12, fzoom = 10;
  int xpos, offset, fontshrink;
  int nsub, nsup, savedpos;
  int csize;
  char symbol[30];

  savedpos = 0;
  switch (zoom_factor)
    {
    case 0:
      fontsize = 8;		/*8 */
      fzoom = 6;		/*6,4 */
      break;
    case 1:
      fontsize = 10;		/*6? */
      fzoom = 7;		/*5? */
      break;
    case 2:
      fontsize = 12;		/*9? */
      fzoom = 8;		/*7? */
      break;
    case 3:
      fontsize = 16;		/*12? */
      fzoom = 9;		/*12,10? */
      break;
    case 4:
      fontsize = 18;
      fzoom = 10;
      break;
    default:
      fprintf (stderr, "no figfont parameters for zoom factor %d\n",
	       zoom_factor);
    }

  if ((fp = fopen (filename, "w")) == NULL)
    return (1);
  fprintf (fp, "#FIG 3.2\n");
  fprintf (fp, "Portrait\n");
  fprintf (fp, "Center\n");
  fprintf (fp, "Metric\n");
  fprintf (fp, "A4\n");
  fprintf (fp, "100.00\n");
  fprintf (fp, "Single\n");
  fprintf (fp, "0\n");
  fprintf (fp, "# generated by Chemtool " VERSION "\n");
  fprintf (fp, "1200 2\n");

  figline.object = 2;
  figline.sub_type = 1;
  figline.line_style = 0;
  figline.thickness = 2;
  figline.pen_color = 0;
  figline.fill_color = 7;
  figline.depth = 100;
  figline.pen_style = 0;
  figline.area_fill = -1;
  figline.style_val = 0.000;
  figline.join_style = 0;
  figline.cap_style = 0;
  figline.radius = -1;
  figline.forward_arrow = 0;
  figline.backward_arrow = 0;
  figline.npoints = 2;

  hp_b = da_root.next;
  for (d = 0; d < hp.n; d++)
    {
    if ( mark.flag && (hp_b->smarked+hp_b->tmarked)==0 )
    {}else{
      coord = bond_cut (hp_b->x, hp_b->y, hp_b->tx, hp_b->ty, 12);
      x = coord->x * fzoom;
      y = coord->y * fzoom;
      tx = coord->tx * fzoom;
      ty = coord->ty * fzoom;
      if (!hp_b->bond)
	xfig_line (fp, x, y, tx, ty);
      if (hp_b->bond == 5) {
	xfig_wedge (fp, x, y, tx, ty,fzoom,latex_flag);
      }
      if (hp_b->bond == 6)
	{
	  for (i = 0; i < 10; i++)
	    xfig_line (fp, x + 0.1 * i * (tx - x) - 0.01 * (ty - y) * i,
		       y + 0.1 * i * (ty - y) + 0.01 * (tx - x) * i,
		       x + 0.1 * i * (tx - x) + 0.01 * (ty - y) * i,
		       y + 0.1 * i * (ty - y) - 0.01 * (tx - x) * i);
	}
      if (hp_b->bond == 7)
	xfig_wiggly (fp, x, y, tx, ty);
      if (hp_b->bond == 8)
	xfig_arrow (fp, x, y, tx, ty, 0, latex_flag);
      if (hp_b->bond == 9)
	xfig_arrow (fp, x, y, tx, ty, 1, latex_flag);
      if (hp_b->bond == 11)
	xfig_circle (fp, x, y, tx, ty);
      if (hp_b->bond == 10)
	{
	  figline.thickness = 6;	/* 4 ? */
	  xfig_line (fp, x, y, tx, ty);
	  figline.thickness = 2;	/*  1 ? */
	}
      if (hp_b->bond == 12)
	{
	  figline.line_style = 2;
	  figline.style_val = 4;
	  xfig_line (fp, x, y, tx, ty);
	  figline.line_style = 0;
	  figline.style_val = 0;
	}
      if (hp_b->bond == 13)
	{
	  figline.thickness = 6;
	  figline.pen_color = 7;
	  figline.depth = 85;
	  xfig_line (fp, x + (tx - x) / 4, y + (ty - y) / 4,
		     tx - (tx - x) / 4, ty - (ty - y) / 4);
	  figline.thickness = 2;
	  figline.pen_color = 0;
	  figline.depth = 80;
	  xfig_line (fp, x, y, tx, ty);
	  figline.depth = 100;
	}
      if (hp_b->bond == 4)
	{
	  coord = center_double_bond (x, y, tx, ty, 4 * fzoom);
	  xfig_line (fp, coord->x, coord->y, coord->tx, coord->ty);
	  coord++;
	  xfig_line (fp, coord->x, coord->y, coord->tx, coord->ty);
	}
      if (hp_b->bond == 1 || hp_b->bond == 3)
	{
	  coord = multi_bonds (x, y, tx, ty, 10 * fzoom);
	  xfig_line (fp, x, y, tx, ty);
	  xfig_line (fp, coord->x, coord->y, coord->tx, coord->ty);
	}
      if (hp_b->bond == 2 || hp_b->bond == 3)
	{
	  coord = multi_bonds (tx, ty, x, y, 10 * fzoom);
	  xfig_line (fp, x, y, tx, ty);
	  xfig_line (fp, coord->x, coord->y, coord->tx, coord->ty);
	}
      if (hp_b->bond == 14 )
      {
        coord = multi_bonds (x, y, tx, ty, 10 * fzoom);
        xfig_line (fp, x, y, tx, ty);
        figline.line_style=1;
        figline.style_val=2.;
        xfig_line (fp, coord->x, coord->y, coord->tx, coord->ty);
        figline.line_style=0;
        figline.style_val=0.;
      }
      if (hp_b->bond == 15)
      {
        coord = multi_bonds (tx, ty, x, y, 10 * fzoom);
        xfig_line (fp, x, y, tx, ty);
        figline.line_style=1;
        figline.style_val=2.;
        xfig_line (fp, coord->x, coord->y, coord->tx, coord->ty);
        figline.line_style=0;
        figline.style_val=0.;
      }
      if (hp_b->bond == 16)
      {
        coord = center_double_bond (x, y, tx, ty, 4 * fzoom);
	x=coord->x;
	y=coord->y;
	tx=coord->tx;
	ty=coord->ty;
	coord++;
	  for (i = 0; i < 10; i++)
	    xfig_line (fp, x + 0.1 * i * (tx - x),
		       y + 0.1 * i * (ty - y),
		       coord->x + 0.1 * i * (coord->tx - coord->x),
		       coord->y + 0.1 * i * (coord->ty - coord->y) ) ;
	}
/*      hp_b = hp_b->next;*/
}
hp_b= hp_b->next;
}
  figtext.object = 4;
  figtext.sub_type = 0;
  figtext.color = 0;
  figtext.depth = 90;
  figtext.pen_style = 0;
  figtext.font = 16;		/*16=helvetica*,12=courier */
  figtext.font_size = fontsize;
  figtext.angle = 0.000;
  figtext.font_flags = 6;
  figtext.height = 135;
  figtext.length = 405;

  ha = 9 * fzoom;

  hp_a = dac_root.next;
  for (d = 0; d < hp.nc; d++)
    {
    if (mark.flag && hp_a->marked  == 0){}
    else{ 
      figtext.sub_type = abs (hp_a->direct);
      switch (hp_a->direct)
	{
	case Middle_Text:
	  chl = 0;
	  break;
	case Left_Text:
	  chl = 9 * fzoom;
	  break;
	case Right_Text:
	  chl = -9 * fzoom;
	  break;
	default:
	  fprintf (stderr, "undefined text direction in xfig output\n");
	  ;;
	}
      nsub = 0;
      nsup = 0;
      for (i = 0; i < strlen (hp_a->c); ++i)
	if (hp_a->c[i] == '\\')
	  hp_a->c[i] = ' ';
      if (!strpbrk (hp_a->c, "@_^|"))
	{			/*no sub- or superscripts */
	  xpos = hp_a->x * fzoom;
	  csize = fontsize * fzoom * 1.4;
	  if (chl < 0)
	    xpos -= strlen (hp_a->c) * csize;
	  if (chl == 0)
	    xpos -= strlen (hp_a->c) * csize / 2;
	  ulx = xpos;
	  uly = hp_a->y * fzoom - ha;
	  lrx = xpos + strlen (hp_a->c) * csize;
	  if (chl<0) lrx = lrx+csize;
	  lry = hp_a->y * fzoom + ha;
/*whiteout box*/
	  fprintf (fp, " 6 %i %i %i %i\n", ulx, uly, lrx, lry);
	if (! latex_flag) {
	  fprintf (fp, " 2 2 0 1 7 7 95 0 20 1 0 0 5 0 0 5\n");
	  fprintf (fp, "%i %i\n", ulx, uly);
	  fprintf (fp, "%i %i\n", lrx, uly);
	  fprintf (fp, "%i %i\n", lrx, lry);
	  fprintf (fp, "%i %i\n", ulx, lry);
	  fprintf (fp, "%i %i\n", ulx, uly);
	}
	  if (latex_flag)
	    {
	      fprintf (fp,
		       "%i %i %i %i %i %i %f %f %i %f %f %i %i %s\\001\n",
		       figtext.object, figtext.sub_type, figtext.color,
		       figtext.depth, figtext.pen_style, figtext.font,
		       figtext.font_size, figtext.angle, figtext.font_flags,
		       figtext.height, figtext.length * strlen (hp_a->c),
		       hp_a->x * fzoom - chl, hp_a->y * fzoom + ha, hp_a->c);
	    }
	  else
	    {
	      fprintf (fp, "%i %i %i %i %i %i %f %f %i %f %f %i %i %s\\001\n",
		       figtext.object, figtext.sub_type, figtext.color,
		       figtext.depth, figtext.pen_style, figtext.font,
		       figtext.font_size, figtext.angle, figtext.font_flags,
		       figtext.height, figtext.length * strlen (hp_a->c),
		       hp_a->x * fzoom - chl, hp_a->y * fzoom + ha, hp_a->c);
	    }
	    fprintf(fp,"-6\n"); /* close block (around whiteout and label)*/
	}
      else
	{			/* special formatting needed, every character becomes a separate entity */
	  l = strlen (hp_a->c);
	  l = 0;
	  for (i = 0; i < strlen (hp_a->c); i++)
	    {
	      switch (hp_a->c[i])
		{
		case '@':
		  break;
		case '_':
		case '^':
/*		  l -= .5; ??*/
		  break;
		default:
		  l++;
		}
	    }
	  xpos = hp_a->x * fzoom - chl;
	  csize = fontsize * fzoom * 1.4;
	  if (chl < 0)
/*	    xpos = hp_a->x * fzoom - (l - 1) * csize;*****/
/*	    xpos = hp_a->x * fzoom - (l - 3) * csize;*****/
	    xpos = hp_a->x * fzoom - (l - 2) * csize;
	  if (chl == 0)
	    xpos = hp_a->x * fzoom - l / 2 * csize;
/* glue text into compound for easier editing in xfig */
	  ulx = xpos;
	  if (chl < 0)
	    ulx -= csize;
	  uly = hp_a->y * fzoom - ha;
	  lrx = xpos + l  * csize;
	  if (chl <0) lrx = xpos + (l - 1) * csize;
	  lry = hp_a->y * fzoom + ha;
	  fprintf (fp, " 6 %i %i %i %i\n", ulx, uly, lrx, lry);
/*whiteout box*/
	if (!latex_flag){
	  fprintf (fp, " 2 2 0 1 7 7 95 0 20 1 0 0 5 0 0 5\n");
	  fprintf (fp, "%i %i\n", ulx, uly);
	  fprintf (fp, "%i %i\n", lrx, uly);
	  fprintf (fp, "%i %i\n", lrx, lry);
	  fprintf (fp, "%i %i\n", ulx, lry);
	  fprintf (fp, "%i %i\n", ulx, uly);
	  }
	  for (i = 0; i < strlen (hp_a->c); i++)
	    {
	      offset = 0;
	      fontshrink = 0;
	      figtext.font = 16;	/*16? */
	      csize = fontsize * fzoom * 1.4;

	      if (hp_a->c[i] == '_')
		{
		  if (chl >= 0)
		    csize = csize / 2;
		  offset = ha / 2;
		  nsub = nsub + 1;
		  if (nsub == 1)
		    {
		      if (nsup > 0)
			xpos = savedpos;
		      else
			savedpos = xpos;
		    }
		  nsup = 0;
		}

	      if (hp_a->c[i] == '^')
		{
		  offset = -ha / 2;
		  nsup = nsup + 1;
		  if (nsup == 1)
		    {
		      if (nsub > 0)
			xpos = savedpos;
		      else
			savedpos = xpos;
		    }
		  nsub = 0;
		}

	      if (offset != 0)
		{
		  if (hp_a->c[i + 1] != '-' && hp_a->c[i + 1] != '+')
		    fontshrink = 3;
		  offset = offset * 2;

		  if (chl < 0)
		    xpos = xpos - csize/2;
		  else
		  xpos = xpos - 20;
/*		    xpos = xpos + chl;*/
		  i++;
		}
	      else
		{		/*reset sub- and superscript counters */
		  nsub = 0;
		  nsup = 0;
		}

	      if (hp_a->c[i] == '@')
		{
		  figtext.font = 32;
		  i++;
		}
	      if (hp_a->c[i] == '|')
		{
		  figtext.font = 17;
		/*  csize = csize / 2;*/
		  i++;
		}

	      if (latex_flag)
		{
		if (figtext.font==32 || (unsigned char)hp_a->c[i]>128) {
			   switch ((unsigned char)hp_a->c[i]) {
			case 'a':
				strcpy(symbol,"\\\\alpha");
				break;
			case 'b':
				strcpy(symbol,"\\\\beta");
				break;
			case 'c':
				strcpy(symbol,"\\\\chi");
				break;
			case 'd':
				strcpy(symbol,"\\\\delta");
				break;
			case 'e':
				strcpy(symbol,"\\\\varepsilon");
				break;
			case 'f':
				strcpy(symbol,"\\\\phi");
				break;
			case 'g':
				strcpy(symbol,"\\\\gamma");
				break;
			case 'h':
				strcpy(symbol,"\\\\eta");
				break;
			case 'i':
				strcpy(symbol,"\\\\iota");
				break;
			case 'j':
				strcpy(symbol,"\\\\varphi");
				break;
			case 'k':
				strcpy(symbol,"\\\\kappa");
				break;
			case 'l':
				strcpy(symbol,"\\\\lambda");
				break;
			case 'm':
				strcpy(symbol,"\\\\mu");
				break;
			case 'n':
				strcpy(symbol,"\\\\nu");
				break;
			case 'p':
				strcpy(symbol,"\\\\pi");
				break;
			case 'q':
				strcpy(symbol,"\\\\theta");
				break;
			case 'r':
				strcpy(symbol,"\\\\rho");
				break;
			case 's':
				strcpy(symbol,"\\\\sigma");
				break;
			case 't':
				strcpy(symbol,"\\\\tau");
				break;
			case 'u':
				strcpy(symbol,"\\\\upsilon");
				break;
			case 'v':
				strcpy(symbol,"\\\\varpi");
				break;
			case 'w':
				strcpy(symbol,"\\\\omega");
				break;
			case 'x':
				strcpy(symbol,"\\\\xi");
				break;
			case 'y':
				strcpy(symbol,"\\\\psi");
				break;
			case 'z':
				strcpy(symbol,"\\\\zeta");
				break;
			case 'C':
				strcpy(symbol,"X");
				break;
			case 'D':
				strcpy(symbol,"\\\\Delta");
				break;
			case 'F':
				strcpy(symbol,"\\\\Phi");
				break;
			case 'G':
				strcpy(symbol,"\\\\Gamma");
				break;
			case 'J':
				strcpy(symbol,"\\\\vartheta");
				break;
			case 'L':
				strcpy(symbol,"\\\\Lambda");
				break;
			case 'P':
				strcpy(symbol,"\\\\Pi");
				break;
			case 'Q':
				strcpy(symbol,"\\\\Theta");
				break;
			case 'R':
				strcpy(symbol,"P");
				break;
			case 'S':
				strcpy(symbol,"\\\\Sigma");
				break;
			case 'U':
				strcpy(symbol,"\\\\Upsilon");
				break;
			case 'V':
				strcpy(symbol,"\\\\varsigma");
				break;
			case 'W':
				strcpy(symbol,"\\\\Omega");
				break;
			case 'X':
				strcpy(symbol,"\\\\Xi");
				break;
			case 'Y':
				strcpy(symbol,"\\\\Psi");
				break;
			case (unsigned char)173:
				strcpy(symbol,"\\\\uparrow");
				break;	
			case (unsigned char)175:
				strcpy(symbol,"\\\\downarrow");
				break;	
			case (unsigned char)174:
				strcpy(symbol,"\\\\bigcirc\\\\hspace{-1.5ex}R");
				break;	
			default:
				fprintf(stderr,"no translation for %d\n",(unsigned char)hp_a->c[i]);
				symbol[0]=hp_a->c[i];
				symbol[1]='\0';
				break;
			}
		  fprintf (fp,
			   "%i %i %i %i %i %i %f %f %i %f %f %i %i %s \\001\n",
			   figtext.object, figtext.sub_type, figtext.color,
			   figtext.depth, figtext.pen_style, figtext.font,
			   figtext.font_size - fontshrink, figtext.angle,
			   figtext.font_flags, figtext.height, figtext.length,
			   xpos + abs (offset / 6),
			   hp_a->y * fzoom + ha + offset, symbol);
			}else{		
		  fprintf (fp,
			   "%i %i %i %i %i %i %f %f %i %f %f %i %i %c\\001\n",
			   figtext.object, figtext.sub_type, figtext.color,
			   figtext.depth, figtext.pen_style, figtext.font,
			   figtext.font_size - fontshrink, figtext.angle,
			   figtext.font_flags, figtext.height, figtext.length,
			   xpos + abs (offset / 6),
			   hp_a->y * fzoom + ha + offset, hp_a->c[i]);
		 }
		}
	      else
		{
		  fprintf (fp,
			   "%i %i %i %i %i %i %f %f %i %f %f %i %i %c\\001\n",
			   figtext.object, figtext.sub_type, figtext.color,
			   figtext.depth, figtext.pen_style, figtext.font,
			   figtext.font_size - fontshrink, figtext.angle,
			   figtext.font_flags, figtext.height, figtext.length,
			   xpos + abs (offset /6), 
			   hp_a->y * fzoom + ha + offset, hp_a->c[i]);
		}
	      xpos = xpos + csize;
		if (zoom_factor<2) xpos = xpos +20;

/*	      if (hp_a->c[i] == 'F' || hp_a->c[i] == 't')
		xpos -= ha;
*/
	    }			/*for i */
	  fprintf (fp, "-6\n");	/* close this compound */
	}
      for (i = 0; i < strlen (hp_a->c); ++i)
	if (hp_a->c[i] == ' ')
	  hp_a->c[i] = '\\';
    }
      hp_a = hp_a->next;
    }

  hp_sp = sp_root.next;
  for (d = 0; d < hp.nsp; d++)
    {
    if (mark.flag == 1&& hp_sp->marked == 0 ) {}
    else {
	xfig_spline(fp, hp_sp->x0*fzoom,hp_sp->y0*fzoom,
		        hp_sp->x1*fzoom,hp_sp->y1*fzoom,
		        hp_sp->x2*fzoom,hp_sp->y2*fzoom,
		        hp_sp->x3*fzoom,hp_sp->y3*fzoom, 
		        hp_sp->type, latex_flag);
	}
	hp_sp=hp_sp->next;
	}	    
  fclose (fp);
  return (0);
}

int
export_xfig (char *filename)
/* initiate exporting of the current molecule in plain XFig format */
{
  char xfile[512];

  if (strlen (filename))
    {
      sprintf (xfile, "%s", filename);
      exfig (xfile, 0);
    }
  return (0);
}

int
export_latex_pic (char *filename, float scale)
/* export molecule as a pictex file by writing a temporary XFig file and
   postprocessing that with fig2dev */
{
  char com[255];
  char xfile[512];
  int rval;

  if (strlen (filename))
    {
      sprintf (com, "fig2dev -Lpictex -m %f %s.f2l %s", scale, filename, filename);
      sprintf (xfile, "%s.f2l", filename);
      exfig (xfile, 1);
      rval = system (com);
      if (!rval)unlink (xfile);
      return (rval);
    }
  else
    return (1);
}

int
export_ps_pic (char *filename,float scale)
/* export molecule as a postscript file by writing a temporary XFig file and
   postprocessing that with fig2dev */
{
  char com[255];
  char xfile[512];
  int rval;

  if (strlen (filename))
    {
      sprintf (com, "fig2dev  -L eps -m %f %s.f2l %s", scale, filename, filename);
      sprintf (xfile, "%s.f2l", filename);
      exfig (xfile, 0);
      rval = system (com);
      if (rval)
	{			/* old versions of fig2dev do not understand -L eps */
	  sprintf (com, "fig2dev -Lps -m %f  %s.f2l %s", scale, filename, filename);
	  rval = system (com);
	}
      unlink (xfile);
      return (rval);
    }
  else
    return (1);
}

int
export_fw (char *filename)
/* save file and postprocessing that with cht */
{
  char com[255];
  char xfile[512];
  int rval;
  FILE *chtpipe;

  if (strlen (filename))
    {
      sprintf (com, "cht %s.rad ", filename);
      sprintf (xfile, "%s.rad", filename);
      save_mol (xfile, mark.flag);
      chtpipe = popen (com, "r");
      fscanf (chtpipe, "%20s %20s %20s %64s", formula, weight, eweight,
	      compos);
      rval = pclose (chtpipe);
      unlink (xfile);
      return (rval);
    }
  else
    return (1);
}
